(ns laser.rasterize)

(import '(java.io File))
(import '(org.apache.commons.io FileUtils))
(import '(javax.imageio ImageIO) )
(import '(javax.swing JFrame))
(import '(java.awt Color BorderLayout))
(import '(ij ImagePlus IJ))
(import '(java.lang Math))

(import '(ij Macro))

(import '(java.io BufferedReader InputStreamReader))
(import '(java.awt.image BufferedImage))

(use 'clojure.contrib.str-utils)

(use 'clojure.contrib.combinatorics)


(use 'clojure.contrib.repl-utils)

(set! *print-length* 20)


(def img "/home/r/graster/test.png")

(defn frame-hash
  "yields a convienent representation for the pixles in an image.
   Because of the size of the structvre generated, this must only be used
   in a transient way so that java can do it's garbage collection."
  [#^java.lang.String image-name]
  (let [image+ (ImagePlus. image-name)]
    (with-meta
      (let [buf (.. image+ getBufferedImage)
	    color (.getColorModel buf)]
	(apply hash-map 
	       (interleave 
		(doall (for [x (range (.getWidth image+)) y (range (.getHeight image+))]
			 (vector x y)))
		(doall (for [x (range (.getWidth image+)) y (range (.getHeight image+))]
			 (let [data (.getRGB buf x y)] 
			   (hash-map :r (bit-shift-right (bit-and 0xff0000 data) 16)
				     :g (bit-shift-right (bit-and 0x00ff00 data) 8)
				     :b (bit-and 0x0000ff data))))))))
      {:width  (.getWidth image+) :height (.getHeight image+)})))


(defn frame-hash->bufferedImage
  [frame-hash]
    (let [data  (meta frame-hash)
	image (BufferedImage. (:width data) (:height data) BufferedImage/TYPE_INT_BGR)]
	
	(doall (for [element frame-hash] 
	  (let [coord  (key element) 
		rgb   (val element)
		packed-RGB 
		(+ (bit-shift-left (:r rgb) 16)
		   (bit-shift-left (:g rgb) 8)
		                   (:b rgb))]
	    (.setRGB image (first coord) (last coord) packed-RGB))))
	image))



